module Texinfo where

import Prelude hiding (head,map,div,span)
import HTMLPrelude
import Example

import Maybe

-- texinfo-style documents

data Node =
  Node	String		-- name of node
	[Node_Content]  -- contents of the node
	[Node]          -- subnodes
  -- administrative fields
	String Int	-- base name & number  of node
	[Int] 		-- section count

type Node_Content = String

tree2html :: Node -> Maybe Node -> String -> IO ()
tree2html node m_up file_stub =
  nextfile (renumber node file_stub) 1 Nothing Nothing m_up

nextfile node@(Node _ _ subnodes file_stub i _) level m_next m_previous m_up =
  do
    writeFile (make_filename file_stub i) $
      show_document $
      node2html node m_next m_previous m_up
    nextfiles subnodes (level+1) Nothing Nothing (Just node)

nextfiles [] level m_next m_previous m_up =
  return ()
nextfiles (node : nodes) level m_next m_previous m_up =
  do
    nextfile node level (listToMaybe nodes) m_previous m_up 
    nextfiles nodes level m_next (Just node) m_up

renumber node file_stub =
  fst (number node 0 [])
  where
    number (Node name nc subnodes _ _ _) i sec_count =
      let (subnodes', i') = number' subnodes (i+1) (1:sec_count) in
	(Node name nc subnodes' file_stub i sec_count, i')

    number' [] i sec_count =
      ([], i)
    number' (node : nodes) i sec_count@(sc:scs) =
      let (node', i') = number node i sec_count in
	let (nodes', i'') = number' nodes i' (sc+1:scs) in
	  (node' : nodes', i'')

-- @node  node-name, next, previous, up

node2html :: Node -> Maybe Node -> Maybe Node -> Maybe Node -> ELT DOCUMENT
node2html (Node name contents subnodes _ _ sec_count) m_next m_previous m_up =
  html_doc title
  (  maybe_link "Next" m_next
  ## maybe_link "Previous" m_previous
  ## maybe_link "Up" m_up
  ## hr empty
  ## pars contents
  ## my_menu node_ref subnodes)
  where
    title = show_sec_count sec_count ++ name
    level = length sec_count

maybe_link link_kind Nothing =
  text ("No " ++ link_kind ++ "; ")
maybe_link link_kind (Just node) =
  text (link_kind ++": ") 
  ## node_ref node
  ## text "; "

my_menu make_ref [] = 
  empty
my_menu make_ref subnodes =
  menu (foldr add_node empty subnodes)
  where
    add_node node items = li (make_ref node) ## items

node_ref (Node name _ _ file_stub i sec_count) =
  a (  text (show_sec_count sec_count ++ name)
    ## attr HREF (make_filename file_stub i))

make_filename stub i =
  stub ++ show i ++ ".html"

pars :: AddTo a P => [String] -> ELT a -> ELT a
pars = foldr (##) empty . Prelude.map (p . text)

{-- 
pars [] =
  empty
pars (str:strs) =
  p (text str)
  ## pars strs
--}

show_sec_count [] = ""
show_sec_count scs = show_sec_count0 scs " "

show_sec_count0 [] fill = ""
show_sec_count0 (sc:scs) fill = show_sec_count0 scs "." ++ show sc ++ fill

-- 

node0 = 
  Node 
   "TOP"
   [ "This is the first paragraph of very interesting text."
   , "And this is the 2nd even more interesting paragraph."]
   [ Node "Introduction"
     ["Here is some introductory text that, I felt, I had to share with everyone"]
     [Node "Sub-intro" 
       ["and we have a sub-introduction, too"][]
       "node" 0 []]
       "node" 2 []
   , Node "Notation"
     ["Hey Jude, don't make it bad. Take a sad song and make it better"][]
     "node" 3 []
   , Node "Give me some meat"
     ["Remember to let her under your skin, then you'll begin to make it better"][]
     "node" 0 []]
   "node" 0 []

example = 
  putStr
  $ show_document
  $ node2html 
    (renumber node0 "mynode")
    (Just $ Node "Sec 1" [] [] "node" 1 [])
    Nothing
    Nothing

example1 file_name_stub = 
  tree2html node0 Nothing file_name_stub
